﻿#ifndef ROOMLIST_H
#define ROOMLIST_H

#include <QMap>
#include <QVector>
#include <QWidget>
#include <QMouseEvent>
#include "KRtcClient.h"
#pragma execution_character_set("utf-8")
using namespace KRTCSDK;
namespace Ui {
class RoomWindow;
}

class QLabel;
class LoginWindow;
class DisplayWidget;
class QListWidgetItem;

class RoomWindow : public QWidget , public KRTCSDK::KRtcCallback
{
    Q_OBJECT

public:
    explicit RoomWindow(QWidget *parent = nullptr);
    ~RoomWindow();

    DisplayWidget* getIdleDisplayWidget();
    DisplayWidget* getDisplayWidget(QString userId);
    void onWarning(RTC_SDK_WARNING warningType, const char* content) override;
    void onError(RTC_SDK_ERROR errorType, const char* content) override;
    void onRelease(int32_t result) override;
    void onLogin(int32_t result, userId id) override;
    void onLogout(int32_t result) override;
    void onJoinRoom(int32_t result) override;
    void onLeaveRoom(int32_t result) override;
    void onIncomingCall(const char* roomId, const char* roomName) override;
    void onCancelCall(const char* roomId, const char* roomName) override;
    void onReplyCall(bool answerOrHanup, const char* roomId, int32_t result) override;
    void onPublishStream(STREAM_TYPE type, uint8_t mediaIndex, int32_t result) override;
    void onUnpublishStream(STREAM_TYPE type, uint8_t mediaIndex, int32_t result) override;
    void onRemoteUserJoinRoom(userId id, const char* nickName) override;
    void onRemoteUserLeaveRoom(userId id) override;
    void onRemoteUserPublishStream(userId id, StreamParam* param) override;
    void onRemoteUserUnpublishStream(userId id, STREAM_TYPE type, uint8_t mediaIndex) override;
    void onLocalVideoStatistics(STREAM_TYPE type, uint8_t mediaIndex, VideoStatistics* state) override;
    void onLocalAudioStatistics(AudioStatistics* state) override;
    void onRemoteVideoStatistics(userId id, STREAM_TYPE type, uint8_t mediaIndex, VideoStatistics* state) override;
    void onRemoteAudioStatistics(userId id, STREAM_TYPE type, AudioStatistics* state) override;
    void onUnpublishStreamNotify(STREAM_TYPE type, uint8_t mediaIndex) override;
    void onSubscribeStream(userId uid, STREAM_TYPE type, uint8_t mediaIndex, int32_t result) override;
    void onUnsubscribeStream(userId uid, STREAM_TYPE type, uint8_t mediaIndex, int32_t result) override;
    void onSnapshotVideo(userId id, STREAM_TYPE type, uint8_t mediaIndex, const TRtcRawVideoData* rgb, int32_t result) override;
    void onAudioControlPlayoutMute(bool playoutMute) override;
    void onAudioControlRecordMute(bool recordMute) override;

signals:
    void sigRelease(int32_t result);
    void sigLogin(int32_t result, QString id);
    void sigLogout(int32_t result);
    void sigJoinRoom(int32_t result);
    void sigLeaveRoom(int32_t result);
    void sigIncomingCall(QString roomId, QString roomName);
    void sigCancelCall(QString roomId, QString roomName);
    void sigReplyCall(bool answerOrHanup, QString roomId, int32_t result);
    void sigPublishStream(STREAM_TYPE, uint8_t, int32_t);
    void sigUnpublishStream(STREAM_TYPE, uint8_t, int32_t);
    void sigRemoteUserJoinRoom(QString id, QString nickName);
    void sigRemoteUserLeaveRoom(QString id);
    void sigRemoteUserPublishStream(QString id, StreamParam param);
    void sigRemoteUserUnPublishStream(QString id, STREAM_TYPE type, uint8_t mediaIndex);
    void sigUnpublishStreamNotify(STREAM_TYPE type, uint8_t mediaIndex);
    void sigSubscribeStream(QString id, STREAM_TYPE type, uint8_t mediaIndex, int32_t result);
    void sigUnsubscribeStream(QString id, STREAM_TYPE type, uint8_t mediaIndex, int32_t result);
    void sigSaveImage(QImage image);
public slots:
    void onSigRelease(int32_t result);
    void onSigLogin(int32_t result, QString id);
    void onSigLogout(int32_t result);
    void onSigJoinRoom(int32_t result);
    void onSigLeaveRoom(int32_t result);
    void onSigIncomingCall(QString roomId, QString roomName);
    void onSigCancelCall(QString roomId, QString roomName);
    void onSigReplyCall(bool answerOrHanup, QString roomId, int32_t result);
    void onSigPublishStream(STREAM_TYPE type, uint8_t mediaIndex, int32_t result);
    void onSigUnpublishStream(STREAM_TYPE type, uint8_t mediaIndex, int32_t result);
    void onSigSubscribeStream(QString id, STREAM_TYPE type, uint8_t mediaIndex, int32_t result);
    void onSigUnsubscribeStream(QString id, STREAM_TYPE type, uint8_t mediaIndex, int32_t result);
    void onSigRemoteUserJoinRoom(QString id, QString nickName);
    void onSigRemoteUserLeaveRoom(QString id);
    void onSigRemoteUserPublishStream(QString id, StreamParam param);
    void onSigRemoteUserUnPublishStream(QString id, STREAM_TYPE type, uint8_t mediaIndex);

    void onSigUnsubscribe();
    void onSigUnpublishStreamNotify(STREAM_TYPE type, uint8_t mediaIndex);
    void onSigSnapshotVideo(QString id, STREAM_TYPE type, uint8_t mediaIndex);
    void onSigSaveImage(QImage image);

protected:
    void mousePressEvent(QMouseEvent *event) override;
    void mouseMoveEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;
    void showEvent(QShowEvent *event) override;
    void hideEvent(QHideEvent *event) override;

private slots:
    void on_quit_clicked();
    void on_mic_clicked();
    void on_speaker_clicked();
    void on_camera_clicked();
    void onSigLogin(QString serverAddr, QString userId, QString nickName, QString roomId, QString appId, QString appSecret);
    void on_screenShare_clicked();
    void on_userList_itemDoubleClicked(QListWidgetItem *item);
    void on_btnSetup_clicked();

    void on_mixSwitchCheckBox_stateChanged(int arg1);

private:
    void changeAudioPattern(int state);
    void unpublishStreamHandle(STREAM_TYPE type);

private:
    Ui::RoomWindow *ui;
    QString m_serverAddr;
    std::string m_roomId;
    QPoint m_dragPosition;
    bool m_bIsDrag{ false };
    LoginWindow* m_loginWindow;
    KRTCSDK::KRtcClient* m_client;
    KRTCSDK::KRtcDeviceManager* m_deviceManager;
    QMap<QString, QMap<STREAM_TYPE, StreamParam>> m_remoteUserStream;

    DisplayWidget* m_view[3];
    int m_index{ 0 };  // 用新的流顶替掉之前的
    bool micEnable{ false };
    bool spkEnable{ true };
    bool cameraEnable{ false };
    bool screenShare{ false };
    bool hasSubscribedVmp{ false };
    bool hasSubscribedMix{ false };
    bool hasSubscribedDual{ false };
};

#endif // ROOMLIST_H
