﻿#include <QDebug>
#include "KRtcClient.h"
#include "devicelistwindow.h"
#include "KRtcDeviceManager.h"
#include "ui_devicelistwindow.h"
using namespace KRTCSDK;
DeviceListWindow::DeviceListWindow(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DeviceListWindow)
{
    ui->setupUi(this);

    this->m_deviceMgr = KRTCSDK::getDeviceManager();

    this->setWindowTitle("设置");
    this->setAttribute(Qt::WA_DeleteOnClose);

    DeviceList* micList = m_deviceMgr->getDeviceList(DEVICETYPE_MIC);
    QIcon micIcon(":/icon/mic-open.png");
    for (int i = 0; i < micList->count; ++i)
    {
        ui->micList->addItem(micIcon, micList->devices[i].deviceName, QString(micList->devices[i].deviceId));
    }
    m_deviceMgr->releaseDeviceList(micList);

    DeviceList* spkList = m_deviceMgr->getDeviceList(DEVICETYPE_SPEAKER);
    QIcon spkIcon(":/icon/spk-open.png");
    for (int i = 0; i < spkList->count; ++i)
    {
        ui->spkList->addItem(spkIcon, spkList->devices[i].deviceName, QString(spkList->devices[i].deviceId));
    }
    m_deviceMgr->releaseDeviceList(spkList);

    DeviceList* cameraList = m_deviceMgr->getDeviceList(DEVICETYPE_CAMERA);
    if (cameraList != nullptr && cameraList->count > 0)
    {
        QIcon cameraIcon(":/icon/camera-open.png");

        // 先添加 currentCamera 防止预览的 camera 与发布流使用的 camera 不同导致切换
        DeviceInfo* cameraInfo = m_deviceMgr->getCurrentDevice(DEVICETYPE_CAMERA);
        ui->cameraList->addItem(cameraIcon, cameraInfo->deviceName, QString(cameraInfo->deviceId));

        for (int i = 0; i < cameraList->count; ++i)
        {
            if (strcmp(cameraList->devices[i].deviceId, cameraInfo->deviceId) != 0)
            {
                ui->cameraList->addItem(cameraIcon, cameraList->devices[i].deviceName, QString(cameraList->devices[i].deviceId));
            }
        }

        if (cameraList->count > 0)
        {
            ui->displayWidget->setUpdatesEnabled(false);
        }

        this->m_deviceMgr->startCameraDeviceTest(cameraInfo->deviceId, (void*)ui->displayWidget->winId(), false);

        m_deviceMgr->releaseDeviceInfo(cameraInfo);
        m_deviceMgr->releaseDeviceList(cameraList);
    }
}

DeviceListWindow::~DeviceListWindow()
{
    delete ui;
}

void DeviceListWindow::closeEvent(QCloseEvent *)
{
    m_deviceMgr->stopCameraDeviceTest();
}

void DeviceListWindow::on_micList_currentIndexChanged(int index)
{
    qDebug() << __FUNCTION__;
    this->m_deviceMgr->setCurrentDevice(DEVICETYPE_MIC, ui->micList->itemData(index).toString().toStdString().c_str());
}

void DeviceListWindow::on_spkList_currentIndexChanged(int index)
{
    qDebug() << __FUNCTION__;
    this->m_deviceMgr->setCurrentDevice(DEVICETYPE_SPEAKER, ui->spkList->itemData(index).toString().toStdString().c_str());
}

void DeviceListWindow::on_cameraList_currentIndexChanged(int index)
{
    qDebug() << __FUNCTION__;
    std::string deviceId = ui->cameraList->itemData(index).toString().toStdString();
    this->m_deviceMgr->setCurrentDevice(DEVICETYPE_CAMERA, deviceId.c_str());
}
