﻿#include <QPainter>
#include <QResizeEvent>
#include "displaywidget.h"
#include "ui_displaywidget.h"

DisplayWidget::DisplayWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::DisplayWidget)
{
    ui->setupUi(this);
    ui->btnPauseOrResume->hide();
    ui->btnDecrease->hide();
    ui->btnIncrease->hide();
    ui->btnDelete->hide();
    ui->btnGrabpic->hide();

    setAttribute(Qt::WA_StyledBackground);
}

DisplayWidget::~DisplayWidget()
{
    delete ui;
}

void DisplayWidget::subscribeStream(QString id, STREAM_TYPE type, uint8_t mediaIndex)
{
    this->m_subscribing = true;
    this->m_subUserId = id;
    this->m_subStreamType = type;
    this->m_mediaIndex = mediaIndex;
    this->ui->cameraIcon->hide();
    this->ui->widget->setUpdatesEnabled(false);
    if (type == STREAM_TYPE_DUAL_VIDEO)
        this->m_titleName = id + "的共享内容";
    else if (type == STREAM_TYPE_VMP)
        this->m_titleName = "画面合成";
    else
        this->m_titleName = id;
    this->ui->userName->setText(this->m_titleName);
}

void DisplayWidget::unSubscribeUser()
{
    this->m_subscribing = false;
    this->m_subUserId.clear();
    this->m_titleName.clear();
    this->ui->userName->clear();
    this->ui->cameraIcon->show();
    this->ui->cameraIcon->setText("");
    this->ui->widget->setUpdatesEnabled(true);
}

void *DisplayWidget::getWinId()
{
    return (void*)(this->ui->widget->winId());
}

void DisplayWidget::enterEvent(QEvent */*event*/)
{
    if (this->m_subscribing)
    {
        ui->btnPauseOrResume->show();
        ui->btnDecrease->show();
        ui->btnIncrease->show();
        ui->btnDelete->show();
        ui->btnGrabpic->show();
    }
}

void DisplayWidget::leaveEvent(QEvent *)
{
    ui->btnPauseOrResume->hide();
    ui->btnDecrease->hide();
    ui->btnIncrease->hide();
    ui->btnDelete->hide();
    ui->btnGrabpic->hide();
}


void DisplayWidget::on_btnDelete_clicked()
{
    emit this->sigUnsubscribe();
}

void DisplayWidget::on_btnPauseOrResume_clicked()
{
    if (this->m_pauseOrResume)
    {
        this->ui->btnPauseOrResume->setStyleSheet("QPushButton{border-image: url(\":/icon/pause-black.png\");}"
                                                  "QPushButton:hover{border-image: url(\":/icon/pause-blue.png\");}"
                                                  "QPushButton:pressed{border-image: url(\":/icon/pause-blue-select.png\");}");
    }
    else
    {
        this->ui->btnPauseOrResume->setStyleSheet("QPushButton{border-image: url(\":/icon/resume-black.png\");}"
                                                  "QPushButton:hover{border-image: url(\":/icon/resume-blue.png\");}"
                                                  "QPushButton:pressed{border-image: url(\":/icon/resume-blue-select.png\");}");
    }
    this->m_pauseOrResume = !this->m_pauseOrResume;
}

void DisplayWidget::on_btnGrabpic_clicked()
{
    emit sigSnapshotVideo(m_subUserId,m_subStreamType,m_mediaIndex);
}
