﻿#include <QDir>
#include <QDebug>
#include <QMovie>
#include <QTextStream>
#include <QGraphicsDropShadowEffect>
#include "KRtcClient.h"
#include "AccessToken.h"
#include "loginwindow.h"
#include "ui_loginwindow.h"
#include "devicelistwindow.h"

LoginWindow::LoginWindow(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::LoginWindow)
{
    ui->setupUi(this);

    // 设置窗口属性
    this->setWindowFlags(Qt::FramelessWindowHint);                // 去掉标题栏
    this->setAttribute(Qt::WA_TranslucentBackground);             // 设置窗口透明

    // 设置阴影
    this->m_shadowEffect = new QGraphicsDropShadowEffect(this);
    this->m_shadowEffect->setOffset(8,8);
    this->m_shadowEffect->setColor(QColor(0,0,0,90));
    this->m_shadowEffect->setBlurRadius(10);
    ui->frame->setGraphicsEffect(m_shadowEffect);

    ui->nickName->setPlaceholderText(tr("请输入你的名字"));
    ui->nickName->addAction(QIcon("://icon/nickName.png"), QLineEdit::LeadingPosition);
    ui->userE164->setPlaceholderText(tr("用户id(匿名登录时无需填写)"));
    ui->userE164->addAction(QIcon("://icon/id.png"), QLineEdit::LeadingPosition);
    ui->serverAddr->setPlaceholderText(tr("请输入服务器地址"));
    ui->serverAddr->addAction(QIcon("://icon/server.png"), QLineEdit::LeadingPosition);
    ui->roomId->setPlaceholderText(tr("请输入房间id"));
    ui->roomId->addAction(QIcon(":/icon/room.png"), QLineEdit::LeadingPosition);
    ui->appId->setPlaceholderText(tr("请输入appid"));
    ui->appId->addAction(QIcon(":/icon/appId.png"), QLineEdit::LeadingPosition);
    ui->appSecret->setPlaceholderText(tr("app secret"));
    ui->appSecret->addAction(QIcon(":/icon/appSecret.png"), QLineEdit::LeadingPosition);

    // 加载动画
    QMovie* loadMovie = new QMovie(":/icon/loading.gif");
    loadMovie->setScaledSize(QSize(40,40));
    loadMovie->start();
    ui->labLoad->setMovie(loadMovie);
    ui->labLoad->hide();

    // 错误提示
    ui->labPrompt->hide();

    QString curPath = QDir::currentPath();
    this->m_loginFile = new QFile(curPath + "/loginInfo");
    if (m_loginFile->exists() && m_loginFile->open(QIODevice::ReadWrite))
    {
        QTextStream textStream(m_loginFile);
        ui->nickName->setText(textStream.readLine());
        ui->userE164->setText(textStream.readLine());
        ui->serverAddr->setText(textStream.readLine());
        ui->roomId->setText(textStream.readLine());
        ui->appId->setText(textStream.readLine());
        ui->appSecret->setText(textStream.readLine());
        m_loginFile->close();
    }
}

LoginWindow::~LoginWindow()
{
    delete ui;
}

void LoginWindow::setPromptText(QString text)
{
    ui->labPrompt->setText(text);
    ui->labPrompt->show();
}

void LoginWindow::onJoinRoomFailed()
{
    this->ui->btnLogin->show();
    this->ui->labLoad->hide();
}

void LoginWindow::mousePressEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton)
    {
        m_bIsDrag = true;
        m_dragPosition = event->globalPos()-frameGeometry().topLeft();
    }
}

void LoginWindow::mouseMoveEvent(QMouseEvent *event)
{
    if (m_bIsDrag)
    {
        move(event->globalPos() - m_dragPosition);
    }
}

void LoginWindow::mouseReleaseEvent(QMouseEvent *event)
{
    m_bIsDrag = false;
}

void LoginWindow::showEvent(QShowEvent *event)
{
    this->ui->btnLogin->show();
    this->ui->labLoad->hide();
}

void LoginWindow::on_btnQuit_clicked()
{
    QApplication::quit();
}

void LoginWindow::on_btnLogin_clicked()
{
    ui->labPrompt->clear();
    ui->labPrompt->hide();

    qDebug("[%s]", __FUNCTION__);
    QString serverAddr = ui->serverAddr->text();
    QString id         = ui->userE164->text();
    QString nickName   = ui->nickName->text();
    QString roomId     = ui->roomId->text();
    QString appId      = ui->appId->text();
    QString appSecret  = ui->appSecret->text();

    if (!serverAddr.isEmpty() && !nickName.isEmpty() && !roomId.isEmpty() &&
        !appId.isEmpty() && !appSecret.isEmpty())
    {
        qDebug("[%s] emit signal login", __FUNCTION__);
        emit this->sigLogin(serverAddr, id, nickName, roomId, appId, appSecret);

        this->ui->btnLogin->hide();
        this->ui->labLoad->show();

        if (m_loginFile->open(QIODevice::WriteOnly))
        {
            QTextStream textStream(m_loginFile);
            textStream << ui->nickName->text()   << "\n";
            textStream << ui->userE164->text()   << "\n";
            textStream << ui->serverAddr->text() << "\n";
            textStream << ui->roomId->text()     << "\n";
            textStream << ui->appId->text()      << "\n";
            textStream << ui->appSecret->text()  << "\n";
            m_loginFile->close();
        }
    }
}

void LoginWindow::on_btnSetup_clicked()
{
    DeviceListWindow* widget = new DeviceListWindow(this);
    widget->open();
}
