﻿#include <QDebug>
#include <QTextCodec>
#include <QListWidgetItem>
#include "roomwindow.h"
#include "loginwindow.h"
#include "AccessToken.h"
#include "ui_roomwindow.h"
#include "devicelistwindow.h"
#include <QDateTime>
#include <QMessageBox>
#include <QDir>


RoomWindow::RoomWindow(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::RoomWindow)
{
    this->m_client = KRTCSDK::createRtcInstance();
    this->m_client->init(this);
    this->m_deviceManager = KRTCSDK::getDeviceManager();

    ui->setupUi(this);

    this->setWindowFlags(Qt::FramelessWindowHint);                // 去掉标题栏

    ui->userList->setIconSize(QSize(26,26));
    ui->userList->setGridSize(QSize(ui->userList->size().width(),50));
    ui->userList->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    this->m_loginWindow = new LoginWindow();
    connect(this->m_loginWindow, SIGNAL(sigLogin(QString, QString, QString, QString, QString, QString)), this, SLOT(onSigLogin(QString, QString, QString, QString, QString, QString)));
    m_loginWindow->show();

    this->m_view[0] = this->ui->view0;
    this->m_view[1] = this->ui->view1;
    this->m_view[2] = this->ui->view2;

    for (int i = 0; i < 3; ++i)
    {
        connect(this->m_view[i], SIGNAL(sigUnsubscribe()),this, SLOT(onSigUnsubscribe()));
        connect(this->m_view[i], &DisplayWidget::sigSnapshotVideo,this, &RoomWindow::onSigSnapshotVideo);
    }

    qRegisterMetaType<uint8_t>("uint8_t");
    qRegisterMetaType<int32_t>("int32_t");
    qRegisterMetaType<STREAM_TYPE>("STREAM_TYPE");
    qRegisterMetaType<StreamParam>("StreamParam");
    qRegisterMetaType<SubscribeStreamParam>("SubscribeStreamParam");

    connect(this, SIGNAL(sigRelease(int32_t)), this, SLOT(onSigRelease(int32_t)));
    connect(this, SIGNAL(sigLogin(int32_t, QString)), this, SLOT(onSigLogin(int32_t, QString)));
    connect(this, SIGNAL(sigLogout(int32_t)), this, SLOT(onSigLogout(int32_t)));
    connect(this, SIGNAL(sigJoinRoom(int32_t)), this, SLOT(onSigJoinRoom(int32_t)));
    connect(this, SIGNAL(sigLeaveRoom(int32_t)), this, SLOT(onSigLeaveRoom(int32_t)));
    connect(this, SIGNAL(sigIncomingCall(QString, QString)), this, SLOT(onSigIncomingCall(QString, QString)));
    connect(this, SIGNAL(sigCancelCall(QString, QString)), this, SLOT(onSigCancelCall(QString, QString)));
    connect(this, SIGNAL(sigReplyCall(bool, QString, int32_t)), this, SLOT(onSigReplyCall(bool, QString, int32_t)));
    connect(this, SIGNAL(sigPublishStream(STREAM_TYPE, uint8_t, int32_t)), this, SLOT(onSigPublishStream(STREAM_TYPE, uint8_t, int32_t)));
    connect(this, SIGNAL(sigUnpublishStream(STREAM_TYPE, uint8_t, int32_t)), this, SLOT(onSigUnpublishStream(STREAM_TYPE, uint8_t, int32_t)));
    connect(this, SIGNAL(sigRemoteUserJoinRoom(QString, QString)), this, SLOT(onSigRemoteUserJoinRoom(QString, QString)));
    connect(this, SIGNAL(sigRemoteUserLeaveRoom(QString)), this, SLOT(onSigRemoteUserLeaveRoom(QString)));
    connect(this, SIGNAL(sigRemoteUserPublishStream(QString, StreamParam)), this, SLOT(onSigRemoteUserPublishStream(QString, StreamParam)));
    connect(this, SIGNAL(sigRemoteUserUnPublishStream(QString, STREAM_TYPE, uint8_t)), this, SLOT(onSigRemoteUserUnPublishStream(QString, STREAM_TYPE, uint8_t)));
    connect(this, SIGNAL(sigUnpublishStreamNotify(STREAM_TYPE, uint8_t)), this, SLOT(onSigUnpublishStreamNotify(STREAM_TYPE, uint8_t)));
    connect(this, SIGNAL(sigSubscribeStream(QString, STREAM_TYPE, uint8_t, int32_t)),
            this, SLOT(onSigSubscribeStream(QString, STREAM_TYPE, uint8_t, int32_t)));
    connect(this, SIGNAL(sigUnsubscribeStream(QString, STREAM_TYPE, uint8_t, int32_t)),
            this, SLOT(onSigUnsubscribeStream(QString, STREAM_TYPE, uint8_t, int32_t)));
    connect(this, &RoomWindow::sigSaveImage, this, &RoomWindow::onSigSaveImage);
}

RoomWindow::~RoomWindow()
{
    KRTCSDK::destroyRtcInstance(this->m_client);

    delete this->ui;
    delete this->m_loginWindow;
}

DisplayWidget *RoomWindow::getIdleDisplayWidget()
{
    for (int i = 0; i < 3; ++i)
    {
        if (!this->m_view[i]->isSubscribing())
        {
            return this->m_view[i];
        }
    }

    return nullptr;
}

DisplayWidget *RoomWindow::getDisplayWidget(QString userId)
{
    for (int i = 0; i < 3; ++i)
    {
        if (this->m_view[i]->isSubscribing() && this->m_view[i]->getSubscribeUser() == userId)
        {
            return this->m_view[i];
        }
    }

    return nullptr;
}

void RoomWindow::onWarning(RTC_SDK_WARNING warningType, const char *content)
{
     qDebug("[%s]", __FUNCTION__);
}

void RoomWindow::onError(RTC_SDK_ERROR errorType, const char *content)
{
     qDebug("[%s]", __FUNCTION__);
}

void RoomWindow::onRelease(int32_t result)
{
     qDebug("[%s]", __FUNCTION__);
}

void RoomWindow::onLogin(int32_t result, userId id)
{
    qDebug("[%s] result:%d, id:%s", __FUNCTION__, result, id);
    emit sigLogin(result, id);
}

void RoomWindow::onLogout(int32_t result)
{
    qDebug("[%s] result:%d", __FUNCTION__, result);
    emit sigLogout(result);
}

void RoomWindow::onJoinRoom(int32_t result)
{
    qDebug("[%s] result:%d", __FUNCTION__, result);
    emit sigJoinRoom(result);
}

void RoomWindow::onLeaveRoom(int32_t result)
{
    qDebug("[%s] result:%d", __FUNCTION__, result);
    emit sigLeaveRoom(result);
}

void RoomWindow::onIncomingCall(const char *roomId, const char *roomName)
{
    qDebug("[%s] roomId:%s, roomName:%s", __FUNCTION__, roomId, roomName);
    emit sigIncomingCall(roomId, roomName);
}

void RoomWindow::onCancelCall(const char *roomId, const char *roomName)
{
    qDebug("[%s] roomId:%s, roomName:%s", __FUNCTION__, roomId, roomName);
    emit sigCancelCall(roomId, roomName);
}

void RoomWindow::onReplyCall(bool answerOrHanup, const char *roomId, int32_t result)
{
    if (result == SDK_NO_ERROR)
    {
        qDebug("[%s] answerOrHanup:%d success", __FUNCTION__, answerOrHanup);
    }
    else
    {
        qDebug("[%s] answerOrHanup:%d failed, error code:%d", __FUNCTION__, answerOrHanup, result);
    }
    emit sigReplyCall(answerOrHanup, roomId, result);
}

void RoomWindow::onPublishStream(STREAM_TYPE type, uint8_t mediaIndex, int32_t result)
{
    emit sigPublishStream(type, mediaIndex, result);
}

void RoomWindow::onUnpublishStream(STREAM_TYPE type, uint8_t mediaIndex, int32_t result)
{
    emit sigUnpublishStream(type, mediaIndex, result);
}

void RoomWindow::onRemoteUserJoinRoom(userId id, const char *nickName)
{
    qDebug("[%s] id:%s, nickName:%s", __FUNCTION__, id, nickName);
    emit sigRemoteUserJoinRoom(id, nickName);
}

void RoomWindow::onRemoteUserLeaveRoom(userId id)
{
    qDebug("[%s] id:%s", __FUNCTION__, id);
    emit sigRemoteUserLeaveRoom(id);
}

void RoomWindow::onRemoteUserPublishStream(userId id, StreamParam *param)
{
    qDebug("[%s] id:%s, type:%d", __FUNCTION__, id, param->type);
    emit sigRemoteUserPublishStream(id, *param);
}

void RoomWindow::onRemoteUserUnpublishStream(userId id, STREAM_TYPE type, uint8_t mediaIndex)
{
    qDebug("[%s] id:%s, type:%d", __FUNCTION__, id, type);
    emit sigRemoteUserUnPublishStream(id, type, mediaIndex);
}

void RoomWindow::onLocalVideoStatistics(STREAM_TYPE type, uint8_t mediaIndex, VideoStatistics *state)
{
    qDebug("[%s] type:%d, mediaIndex:%hhu, width:%u, height:%u, frameRate:%u, bitrate:%.2f lossrate:%.2f, freezeCount:%u, totalFreeze:%u",
           __FUNCTION__, type, mediaIndex,
           state->width, state->height, state->frameRate, state->bitrate,
           state->lossRate,state->freezeCount,state->totalFreezesDuration);
}

void RoomWindow::onLocalAudioStatistics(AudioStatistics *state)
{
    qDebug("[%s] bitrate:%.2f lossrate:%.2f", __FUNCTION__, state->bitrate, state->lossRate);
}

void RoomWindow::onRemoteVideoStatistics(userId id, STREAM_TYPE type, uint8_t mediaIndex, VideoStatistics *state)
{
//    qDebug("[%s] id:%s type:%d, mediaIndex:%hhu, width:%u, height:%u, frameRate:%u, bitrate:%.2f lossrate:%.2f, freezeCount:%u, totalFreeze:%u",
//           __FUNCTION__, id, type, mediaIndex,
//           state->width, state->height, state->frameRate, state->bitrate,
//           state->lossRate,state->freezeCount,state->totalFreezesDuration);
}

void RoomWindow::onRemoteAudioStatistics(userId id, STREAM_TYPE type, AudioStatistics *state)
{
    if (type == STREAM_TYPE_AUDIO)
        qDebug("[%s] id:%s bitrate:%.2f lossrate:%.2f", __FUNCTION__, id, state->bitrate, state->lossRate);
    else if (type == STREAM_TYPE_MIX_AUDIO)
        qDebug("[%s] id:%s mix stream bitrate:%.2f lossrate:%.2f", __FUNCTION__, id, state->bitrate, state->lossRate);
}

void RoomWindow::onUnpublishStreamNotify(STREAM_TYPE type, uint8_t mediaIndex)
{
    qDebug("[%s] type:%d", __FUNCTION__, type);
    emit sigUnpublishStreamNotify(type, mediaIndex);
}

void RoomWindow::onSubscribeStream(userId uid, STREAM_TYPE type, uint8_t mediaIndex, int32_t result)
{
    emit sigSubscribeStream(uid, type, mediaIndex, result);
}

void RoomWindow::onUnsubscribeStream(userId uid, STREAM_TYPE type, uint8_t mediaIndex, int32_t result)
{
    emit sigUnsubscribeStream(uid, type, mediaIndex, result);
}

void RoomWindow::onSnapshotVideo(userId id, STREAM_TYPE type, uint8_t mediaIndex, const TRtcRawVideoData* rgb, int32_t result)
{
    if(result == SDK_NO_ERROR){
        // 避免耗时操作
        qDebug("[%s] userId:%s type:%d mediaIndex:%d rgb.videoType:%d  snapshot successed!", __FUNCTION__, id, type, mediaIndex, rgb->videoType);
        QImage image(rgb->data, rgb->width,rgb->height, QImage::Format_RGBA8888);
        emit sigSaveImage(image.copy());
    } else {
        qDebug("[%s] userId:%s type:%d mediaIndex:%d snapshot failed!", __FUNCTION__, id, type, mediaIndex);
    }
}

void RoomWindow::onAudioControlPlayoutMute(bool playoutMute)
{
    qDebug() << __FUNCTION__ << "playoutMute = " << playoutMute;
    if(playoutMute)
    {
        ui->speaker->setStyleSheet("QPushButton{ border-image: url(\":/icon/spk-close.png\"); }"
                               "QPushButton:pressed { border-image: url(\":/icon/spk-close-select.png\"); }");
    }
    else
    {
        ui->speaker->setStyleSheet("QPushButton{ border-image: url(\":/icon/spk-open.png\"); }"
                               "QPushButton:pressed { border-image: url(\":/icon/spk-open-select.png\"); }");
    }

    this->spkEnable = !playoutMute;
}

void RoomWindow::onAudioControlRecordMute(bool recordMute)
{
    qDebug() << __FUNCTION__ << "recordMute = " << recordMute;
    if(recordMute)
    {
        ui->mic->setStyleSheet("QPushButton{ border-image: url(\":/icon/mic-close.png\"); }"
                               "QPushButton:pressed { border-image: url(\":/icon/mic-close-select.png\"); }");
    }
    else
    {
        ui->mic->setStyleSheet("QPushButton{ border-image: url(\":/icon/mic-open.png\"); }"
                               "QPushButton:pressed { border-image: url(\":/icon/mic-open-select.png\"); }");
    }

    this->micEnable = !recordMute;

}

void RoomWindow::onSigRelease(int32_t result)
{

}

void RoomWindow::onSigLogin(int32_t result, QString id)
{
    if (result == SDK_NO_ERROR)
    {
        this->ui->nickName->setText(id);
        JoinRoomParam param;
        param.roomId = this->m_roomId.c_str();
        param.bitrate = 0;
        this->m_client->joinRoom(&param);
    }
    else
    {
        switch (result)
        {
        case SDK_PARAMETER_ERROR:
            this->m_loginWindow->setPromptText("注册失败，请检查参数是否正确");
            this->m_loginWindow->onJoinRoomFailed();
            break;
        case SDK_LOGIN_REPEATED_REGISTRATION_ERROR:
            this->m_loginWindow->setPromptText("重复注册");
            this->m_loginWindow->onJoinRoomFailed();
        case SDK_NETWORK_ERROR:
            this->m_loginWindow->setPromptText("注册失败，网络异常！");
            this->m_loginWindow->onJoinRoomFailed();
            break;
        case SDK_LOGIN_TOKEN_ERROR:
            this->m_loginWindow->setPromptText("token校验失败！");
            this->m_loginWindow->onJoinRoomFailed();
            break;
        default:
            this->m_loginWindow->setPromptText("未知错误！");
            this->m_loginWindow->onJoinRoomFailed();
            break;
        }

        qDebug("[%s] login failed!, result:%d", __FUNCTION__, result);
    }
}

void RoomWindow::onSigLogout(int32_t result)
{
    if (result == SDK_NO_ERROR)
    {
        qDebug("[%s] logout success!, result:%d", __FUNCTION__, result);
        for (int i=0;i<3;++i)
        {
            this->m_view[i]->unSubscribeUser();
        }
    }
    else
    {
        qDebug("[%s] logout failed!, result:%d", __FUNCTION__, result);
    }

    this->hide();
    this->m_loginWindow->show();
}

void RoomWindow::onSigJoinRoom(int32_t result)
{
    if (result == SDK_NO_ERROR)
    {
        // 入会成功
        this->m_loginWindow->hide();
        this->show();
        // 入会后就推送音频流
        this->m_client->publishAudioStream();
    }
    else
    {
        switch (result) {
        case SDK_PARAMETER_ERROR:
            this->m_client->logout();
            this->m_loginWindow->setPromptText("入会失败，请检查参数是否正确");
            this->m_loginWindow->onJoinRoomFailed();
            break;
        case SDK_JOINROOM_REPEATED_JOIN_ERROR:
            this->m_client->logout();
            this->m_loginWindow->setPromptText("重复加入");
            this->m_loginWindow->onJoinRoomFailed();
            break;
        case SDK_JOINROOM_UNREGISTERED_ERROR:
            this->m_client->logout();
            this->m_loginWindow->setPromptText("未注册平台");
            this->m_loginWindow->onJoinRoomFailed();
            break;
        case SDK_NETWORK_ERROR:
            this->m_client->logout();
            this->m_loginWindow->setPromptText("网络异常");
            this->m_loginWindow->onJoinRoomFailed();
            break;
        default:
            break;
        }
    }
}

void RoomWindow::onSigLeaveRoom(int32_t result)
{
    switch (result)
    {
    case SDK_NETWORK_ERROR:
        qDebug("[%s] 网络异常掉线", __FUNCTION__);
        this->m_loginWindow->setPromptText("网络异常掉线");
        break;
    case SDK_NO_ERROR:
        qDebug("[%s] 退会成功", __FUNCTION__);
        break;
    case SDK_LEAVEROOM_PLEASE_OUT_ERROR:
        qDebug("[%s] 被管理员请出房间", __FUNCTION__);
        this->m_loginWindow->setPromptText("被管理员请出房间");
        break;
    case SDK_LEAVEROOM_ROOM_DISSOLVED_ERROR:
        qDebug("[%s] 会议结束", __FUNCTION__);
        this->m_loginWindow->setPromptText("会议结束");
        break;
    default:
        qDebug("[%s] 退会成功", __FUNCTION__);
        break;
    }

    m_client->logout();
}

void RoomWindow::onSigIncomingCall(QString roomId, QString roomName)
{
    // this->m_client->replyCall(true, roomId.toStdString().c_str());
}

void RoomWindow::onSigCancelCall(QString roomId, QString roomName)
{

}

void RoomWindow::onSigReplyCall(bool answerOrHanup, QString roomId, int32_t result)
{
//    if (result == SDK_NO_ERROR)
//    {
//        qDebug("[%s] useritem.username = %s", __FUNCTION__, e164.c_str());
//    }
//    else
//    {
//        qDebug("[%s] reply call failed, result:%d", __FUNCTION__, result);
//    }
}

//远端用户入会处理
void RoomWindow::onSigRemoteUserJoinRoom(QString id, QString nickName)
{
    qDebug("[%s] userId = %s;nickName = %s", __FUNCTION__, id.toUtf8().data(), nickName.toUtf8().data());

    QListWidgetItem* user = new QListWidgetItem(QIcon(":/icon/friend.png"), nickName);
    user->setData(0, id);
    this->ui->userList->addItem(user);
}

//远端用户离会处理
void RoomWindow::onSigRemoteUserLeaveRoom(QString id)
{
    qDebug("[%s] LeaveRoom Userid:%s", __FUNCTION__, id.toStdString().c_str());
    for (int i = 0; i < ui->userList->count(); i++)
    {
        if (ui->userList->item(i)->data(0).toString() == id)
        {
            QListWidgetItem* user = ui->userList->takeItem(i);
            delete user;
        }
    }

    for (int i = 0; i < 3; ++i)
    {
        qDebug("[%s] m_view[%d] isSubscribing:%d, view User:%s",
               __FUNCTION__, i, m_view[i]->isSubscribing(), m_view[i]->getSubscribeUser().toStdString().c_str());
        if (m_view[i]->isSubscribing() && m_view[i]->getSubscribeUser()  == id)
        {
            qDebug("[%s] m_view[%d] unSubscribeUser", __FUNCTION__, i);
            m_view[i]->unSubscribeUser();
        }
    }
}

void RoomWindow::onSigRemoteUserPublishStream(QString id, StreamParam param)
{
    qDebug("[%s] userId:%s, stream type:%d", __FUNCTION__, id.toStdString().c_str(), param.type);
    QString titlelName = id;
    if (param.type == STREAM_TYPE_MAIN_VIDEO)
    {
        DisplayWidget* widget = getIdleDisplayWidget();
        if (widget != nullptr)
        {
            SubscribeStreamParam subParam;
            subParam.type = param.type;
            subParam.layerIndex = param.layerCount - 1;
            subParam.mediaIndex = param.mediaIndex;
            qDebug("[%s] subscribeStream userId:%s, stream type:%d, main", __FUNCTION__, id.toStdString().c_str(), param.type);
            this->m_client->subscribeStream(id.toStdString().c_str(), &subParam, widget->getWinId());
            widget->subscribeStream(id, param.type, param.mediaIndex);

//            //如果终端发布了音频，则订阅
//            if (it != this->m_remoteUserStream.end() && it.value().contains(STREAM_TYPE_AUDIO))
//            {
//                subParam.type = STREAM_TYPE_AUDIO;
//                qDebug("[%s] subscribeStream userId:%s, stream type:audio", __FUNCTION__, id.toStdString().c_str());
//                this->m_client->subscribeStream(id.toStdString().c_str(), &subParam, nullptr);
//            }
        }
        else
        {
            qDebug("[%s] all the Windows are full! ignore.", __FUNCTION__);
        }
    }
    else if (param.type == STREAM_TYPE_DUAL_VIDEO)
    {
        DisplayWidget* widget = getIdleDisplayWidget();
        if (widget != nullptr)
        {
            SubscribeStreamParam subParam;
            subParam.type = param.type;
            subParam.layerIndex = param.layerCount - 1;
            subParam.mediaIndex = param.mediaIndex;
            qDebug("[%s] subscribeStream userId:%s, stream type:%d, dual", __FUNCTION__, id.toStdString().c_str(), param.type);
            //取消订阅双流时也需要匹配titlelName，避免取消了其他流
            titlelName = id + "的共享内容";
            this->m_client->subscribeStream(id.toStdString().c_str(), &subParam, widget->getWinId());
             widget->subscribeStream(id, param.type, param.mediaIndex);
            this->hasSubscribedDual = true;
        }
        else
        {
            qDebug("[%s] all the Windows are full! ignore.", __FUNCTION__);
        }
        //双流当做一个用户添加到用户列表
        QListWidgetItem* user = new QListWidgetItem(QIcon(":/icon/friend.png"), titlelName);
        user->setData(0, titlelName);
        this->ui->userList->addItem(user);
    }
    else if (param.type == STREAM_TYPE_VMP)
    {
        if(!this->hasSubscribedVmp)
        {
            DisplayWidget* widget = getIdleDisplayWidget();
            if (widget != nullptr)
            {
                SubscribeStreamParam subParam;
                subParam.type = param.type;
                subParam.layerIndex = param.layerCount - 1;
                subParam.mediaIndex = param.mediaIndex;
                qDebug("[%s] subscribeStream userId:%s, stream type:%d, vmp", __FUNCTION__, id.toStdString().c_str(), param.type);
                titlelName = "画面合成";
                this->m_client->subscribeStream(id.toStdString().c_str(), &subParam, widget->getWinId());
                widget->subscribeStream(id, param.type, param.mediaIndex);
                this->hasSubscribedVmp = true;
            }
            else
            {
                qDebug("[%s] all the Windows are full! ignore.", __FUNCTION__);
            }
        }
        else
        {
            qDebug("[%s] has Subscribed Vmp, ignore.", __FUNCTION__);
        }
        //画面合成的当做一个用户添加到用户列表
        QListWidgetItem* user = new QListWidgetItem(QIcon(":/icon/friend.png"), titlelName);
        user->setData(0, titlelName);
        this->ui->userList->addItem(user);
    }
    else if (param.type == STREAM_TYPE_AUDIO)
    {
        if(ui->mixSwitchCheckBox->checkState() == Qt::CheckState::Checked)
        {
            qDebug("[%s] mix audio is open, return.", __FUNCTION__);
            return;
        }
        qDebug("[%s] subscribeStream userId:%s, stream type:%d, audio", __FUNCTION__, id.toStdString().c_str(), param.type);
        SubscribeStreamParam subParam;
        subParam.type = STREAM_TYPE_AUDIO;
        this->m_client->subscribeStream(id.toStdString().c_str(), &subParam, nullptr);

    }
    else if(param.type == STREAM_TYPE_MIX_AUDIO)
    {
        if(ui->mixSwitchCheckBox->checkState() == Qt::CheckState::Unchecked)
        {
            qDebug("[%s] mix audio is close, return.", __FUNCTION__);
            return;
        }
        qDebug("[%s] subscribeStream userId:%s, stream type:%d, mix_audio", __FUNCTION__, id.toStdString().c_str(), param.type);
        SubscribeStreamParam subParam;
        subParam.type = STREAM_TYPE_MIX_AUDIO;
        this->m_client->subscribeStream(id.toStdString().c_str(), &subParam, nullptr);
    }

    auto it = this->m_remoteUserStream.find(titlelName);
    if (it == this->m_remoteUserStream.end())
    {
        this->m_remoteUserStream[titlelName][param.type] = param;
        qDebug("[%s] add stream userId:%s, stream type:%d", __FUNCTION__, titlelName.toStdString().c_str(), param.type);
    }
    else if (!it.value().contains(param.type))
    {
        it.value()[param.type] = param;
        qDebug("[%s] update stream userId:%s, stream type:%d", __FUNCTION__, titlelName.toStdString().c_str(), param.type);
    }
}

void RoomWindow::onSigRemoteUserUnPublishStream(QString id, STREAM_TYPE type, uint8_t mediaIndex)
{
    qDebug("[%s] userId:%s, stream type:%d", __FUNCTION__, id.toStdString().c_str(), type);
    if(type == STREAM_TYPE_VMP)
    {
        for (int i = 0; i < 3; ++i)
        {
            if (this->m_view[i]->isSubscribing() && this->m_view[i]->getSubscribeType() == STREAM_TYPE_VMP)
            {
                qDebug("[%s] view:[%d] unsubscribe user:%s", __FUNCTION__, i, id.toStdString().c_str());
    //            std::string strId = id.toStdString();
    //            this->m_client->unsubscribeStream(strId.c_str(), this->m_view[i]->getSubscribeType(), 0);
    //            this->m_client->unsubscribeStream(strId.c_str(), STREAM_TYPE_AUDIO, 0);
                this->hasSubscribedVmp = false;
                this->m_view[i]->unSubscribeUser();
            }
        }
        QString titlelName = "画面合成";
        //从用户列表删除
        for (int i = 0; i < ui->userList->count(); i++)
        {
            if (ui->userList->item(i)->data(0).toString() == titlelName)
            {
                QListWidgetItem* user = ui->userList->takeItem(i);
                delete user;
            }
        }
    }
    else if(type == STREAM_TYPE_DUAL_VIDEO)
    {
        for (int i = 0; i < 3; ++i)
        {
            if (this->m_view[i]->isSubscribing() &&
                this->m_view[i]->getSubscribeType() == STREAM_TYPE_DUAL_VIDEO &&
                this->m_view[i]->getSubscribeUser() == id)
            {
                qDebug("[%s] view:[%d] unsubscribe user:%s", __FUNCTION__, i, id.toStdString().c_str());
    //            std::string strId = id.toStdString();
    //            this->m_client->unsubscribeStream(strId.c_str(), this->m_view[i]->getSubscribeType(), 0);
    //            this->m_client->unsubscribeStream(strId.c_str(), STREAM_TYPE_AUDIO, 0);
                this->hasSubscribedDual = false;
                this->m_view[i]->unSubscribeUser();
            }
        }
        //从用户列表删除
        QString titlelName = id + "的共享内容";
        for (int i = 0; i < ui->userList->count(); i++)
        {
            if (ui->userList->item(i)->data(0).toString() == titlelName)
            {
                QListWidgetItem* user = ui->userList->takeItem(i);
                delete user;
            }
        }
    }
    else if(type == STREAM_TYPE_MAIN_VIDEO)
    {
        for (int i = 0; i < 3; ++i)
        {
            if (this->m_view[i]->getSubscribeType() == STREAM_TYPE_MAIN_VIDEO && this->m_view[i]->getSubscribeUser() == id)
            {
                qDebug("[%s] view:[%d] unsubscribe user:%s", __FUNCTION__, i, id.toStdString().c_str());
    //            std::string strId = id.toStdString();
    //            this->m_client->unsubscribeStream(strId.c_str(), this->m_view[i]->getSubscribeType(), 0);
    //            this->m_client->unsubscribeStream(strId.c_str(), STREAM_TYPE_AUDIO, 0);
                this->m_view[i]->unSubscribeUser();
            }
        }
    }
}

void RoomWindow::onSigUnsubscribe()
{
    DisplayWidget *widget =  qobject_cast<DisplayWidget*>(sender());
    std::string id = widget->getSubscribeUser().toStdString();
    if(widget->getSubscribeType() == STREAM_TYPE_MAIN_VIDEO)
    {
        m_client->unsubscribeStream(id.c_str(), widget->getSubscribeType(), 0);
        m_client->unsubscribeStream(id.c_str(), STREAM_TYPE_AUDIO, 0);
    }
    else if(widget->getSubscribeType() == STREAM_TYPE_VMP)
    {
        m_client->unsubscribeStream("", widget->getSubscribeType(), 0);
    }
    else if(widget->getSubscribeType() == STREAM_TYPE_DUAL_VIDEO)
    {
        m_client->unsubscribeStream(id.c_str(), widget->getSubscribeType(), 0);
    }
    //widget->unSubscribeUser();    //需要取消订阅回调上来后再刷窗口

}

//发布回调处理
void RoomWindow::onSigPublishStream(STREAM_TYPE type, uint8_t mediaIndex, int32_t result)
{
    qDebug("[%s] type:%d, mediaIndex:%hhu, result:%d", __FUNCTION__, type, mediaIndex, result);
    if (result == SDK_NO_ERROR)
    {
        if (type == STREAM_TYPE_MAIN_VIDEO)
        {
            ui->camera->setStyleSheet("QPushButton{ border-image: url(\":/icon/camera-open.png\"); }"
                                      "QPushButton:pressed { border-image: url(\":/icon/camera-open-select.png\"); } }");
            this->cameraEnable = true;
            ui->camera->setEnabled(true);
        }
        else if (type == STREAM_TYPE_DUAL_VIDEO)
        {
            ui->screenShare->setStyleSheet("QPushButton{ border-image: url(\":/icon/screen-share-open.png\"); }"
                                           "QPushButton:pressed { border-image: url(\":/icon/screen-share-open-select.png\"); }");
            this->screenShare = true;
            ui->screenShare->setEnabled(true);
        }
        else if (type == STREAM_TYPE_AUDIO)
        {
            // 音频推流之后设置静哑音初始状态
            this->micEnable    = false;
            this->m_client->setAudioControlRecordMute(true);
            this->spkEnable    = true;
            this->m_client->setAudioControlPlayoutMute(false);
        }
    }
    else
    {
        if (type == STREAM_TYPE_MAIN_VIDEO)
        {
            ui->camera->setEnabled(true);
        }
        else if (type == STREAM_TYPE_DUAL_VIDEO)
        {
            ui->screenShare->setEnabled(true);
        }
        else if (type == STREAM_TYPE_AUDIO)
        {

        }
    }
}

void RoomWindow::onSigUnpublishStream(STREAM_TYPE type, uint8_t mediaIndex, int32_t result)
{
    qDebug("[%s] type:%d, mediaIndex:%hhu, result:%d", __FUNCTION__, type, mediaIndex, result);
    if (result == SDK_NO_ERROR)
    {
        unpublishStreamHandle(type);
    }
    else
    {
        if (type == STREAM_TYPE_MAIN_VIDEO)
        {
            ui->camera->setEnabled(true);
        }
        else if (type == STREAM_TYPE_DUAL_VIDEO)
        {
            ui->screenShare->setEnabled(true);
        }
        else if (type == STREAM_TYPE_AUDIO)
        {
        }
    }
}

void RoomWindow::onSigSubscribeStream(QString id, STREAM_TYPE type, uint8_t mediaIndex, int32_t result)
{
    qDebug("[%s] userid:%s, stream type:%d, mediaIndex:%hhu, errorcode:%d", __FUNCTION__, id.toStdString().c_str(), type, mediaIndex, result);
    if (result == SDK_NO_ERROR)
    {

    }
}

void RoomWindow::onSigUnsubscribeStream(QString id, STREAM_TYPE type, uint8_t mediaIndex, int32_t result)
{
    qDebug("[%s] userid:%s, stream type:%d, mediaIndex:%hhu, errorcode:%d", __FUNCTION__, id.toStdString().c_str(), type, mediaIndex, result);
    if (result == SDK_NO_ERROR)
    {
        QString titleName = id;
        if (type == STREAM_TYPE_MAIN_VIDEO)
        {
            titleName = id;
        }
        else if (type == STREAM_TYPE_DUAL_VIDEO)
        {
            titleName = id + "的共享内容";
            this->hasSubscribedDual = false;
        }
        else if (type == STREAM_TYPE_VMP)
        {
            titleName = "画面合成";
            this->hasSubscribedVmp = false;
        }
        else
        {
            return;
        }
        for (int i = 0; i < 3; ++i)
        {
            qDebug("[%s] m_view[%d] isSubscribing:%d, view User:%s",
                  __FUNCTION__, i, m_view[i]->isSubscribing(), m_view[i]->getSubscribeUser().toStdString().c_str());
            if (m_view[i]->isSubscribing() && m_view[i]->getSubscribeTileName()  == titleName)
            {
                qDebug("[%s] m_view[%d] User:%s, unSubscribeUser", __FUNCTION__, i, titleName.toStdString().c_str());
                m_view[i]->unSubscribeUser();
            }
        }
    }
}

void RoomWindow::mousePressEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton && event->pos().y() <= 40)
    {
        m_bIsDrag = true;
        m_dragPosition = event->globalPos()-frameGeometry().topLeft();
    }
}

void RoomWindow::mouseMoveEvent(QMouseEvent *event)
{
    if (m_bIsDrag)
    {
        move(event->globalPos() - m_dragPosition);
    }
}

void RoomWindow::mouseReleaseEvent(QMouseEvent *event)
{
    m_bIsDrag = false;
}

void RoomWindow::showEvent(QShowEvent *event)
{
    this->cameraEnable = false;
    this->screenShare  = false;
    this->hasSubscribedVmp = false;
    this->hasSubscribedMix = false;
    this->hasSubscribedDual = false;

    DeviceInfo* micInfo = this->m_deviceManager->getCurrentDevice(DEVICETYPE_MIC);
    if (micInfo == nullptr)
    {
    }
    else
    {
        this->m_deviceManager->releaseDeviceInfo(micInfo);
    }
    ui->mic->setStyleSheet("QPushButton{ border-image: url(\":/icon/mic-close.png\"); }"
                           "QPushButton:pressed { border-image: url(\":/icon/mic-close-select.png\"); }");

    DeviceInfo* spkInfo = this->m_deviceManager->getCurrentDevice(DEVICETYPE_SPEAKER);
    if (spkInfo == nullptr)
    {
    }
    else
    {
        this->m_deviceManager->releaseDeviceInfo(spkInfo);
    }
    ui->speaker->setStyleSheet("QPushButton{ border-image: url(\":/icon/spk-open.png\"); }"
                           "QPushButton:pressed { border-image: url(\":/icon/spk-open-select.png\"); }");

    DeviceInfo* cameraInfo = this->m_deviceManager->getCurrentDevice(DEVICETYPE_CAMERA);
    if (cameraInfo == nullptr)
    {
        this->ui->camera->setEnabled(false);
        this->ui->camera->setStyleSheet("QPushButton{ border-image: url(\":/icon/camera-disable.png\"); }");
    }
    else
    {
        this->m_deviceManager->releaseDeviceInfo(cameraInfo);
        ui->camera->setStyleSheet("QPushButton{ border-image: url(\":/icon/camera-close.png\"); }"
                                  "QPushButton:pressed { border-image: url(\":/icon/camera-close-select.png\"); }");
    }

    ui->screenShare->setStyleSheet("QPushButton{ border-image: url(\":/icon/screen-share-close.png\"); }"
                                   "QPushButton:pressed { border-image: url(\":/icon/screen-share-close-select.png\"); }");
}

void RoomWindow::hideEvent(QHideEvent *event)
{
    m_client->stopLocalPreview(ui->localView->getWinId());
    for (int i = 0; i < 3; ++i)
    {
        m_view[i]->unSubscribeUser();
    }
    ui->userList->clear();
}

void RoomWindow::on_quit_clicked()
{
    qDebug("[%s]Click quit!", __FUNCTION__);
    this->m_client->logout();
//    QApplication::quit();
}

void RoomWindow::on_mic_clicked()
{
    if (this->micEnable)
    {
        this->m_client->setAudioControlRecordMute(true);
    }
    else
    {
        this->m_client->setAudioControlRecordMute(false);
    }
}

void RoomWindow::on_speaker_clicked()
{
    if (this->spkEnable)
    {
        this->m_client->setAudioControlPlayoutMute(true);
    }
    else
    {
        this->m_client->setAudioControlPlayoutMute(false);
    }
}

void RoomWindow::on_camera_clicked()
{
    ui->camera->setEnabled(false);

    if (this->cameraEnable)
    {
        this->m_client->stopLocalPreview(this->ui->localView->getWinId());
        this->m_client->unpublishStream(STREAM_TYPE_MAIN_VIDEO, 0);
    }
    else
    {
        this->m_client->startLocalPreview(this->ui->localView->getWinId());
        this->m_client->publishMainVideoStream(0);
    }
}

void RoomWindow::onSigLogin(QString serverAddr, QString userId, QString nickName, QString roomId, QString appId, QString appSecret)
{
    qDebug() << "[" << __FUNCTION__ << "] serverAddr:" << serverAddr << ", userId" << userId << ", nickName" << nickName << ", roomId:" << roomId
             << ", appId" << appId << ", appSecret:" << appSecret;
    //trimmed去掉前后空格
    std::string strServerAddr = serverAddr.trimmed().toStdString();
    std::string strId         = userId.trimmed().toStdString();
    std::string strNickName   = nickName.trimmed().toStdString();
    std::string strAppId      = appId.trimmed().toStdString();
    std::string strAppSecret  = appSecret.trimmed().toStdString();
    this->m_roomId            = roomId.toStdString();
    if (!strServerAddr.empty() && !strNickName.empty() && !this->m_roomId.empty())
    {

        std::string token = AccessToken::Build(strAppId.c_str(), strAppSecret.c_str(), strId.c_str(), 100000000);

        LoginParam param;
        param.serverAddr = strServerAddr.c_str();
        param.id = strId.c_str();
        param.token = token.c_str();
        param.nickName = strNickName.c_str();
        m_client->login(&param);
    }
}

void RoomWindow::on_screenShare_clicked()
{
    ui->screenShare->setEnabled(false);

    if (this->screenShare)
    {
        this->m_client->unpublishStream(STREAM_TYPE_DUAL_VIDEO, 0);
    }
    else
    {
        ScreenList* list = KRTCSDK::getDeviceManager()->getScreenList();
        if (list->count == 0)
        {
            qDebug("[%s] no available screen!", __FUNCTION__);
            ui->screenShare->setEnabled(true);
            KRTCSDK::getDeviceManager()->releaseScreenList(list);
            return;
        }
        this->m_client->publishDualVideoStream(list->screenId[0]);
        KRTCSDK::getDeviceManager()->releaseScreenList(list);
    }
}

void RoomWindow::on_userList_itemDoubleClicked(QListWidgetItem *item)
{
    QString titleName = item->data(0).toString();
    for (int i = 0; i < 3; ++i)
    {
        if (m_view[i]->getSubscribeTileName() == titleName)
        {
            qDebug() << "[" << __FUNCTION__ << "] user:" << titleName << " already subscribed";
            return;
        }
    }

    auto remoteUserStreamIt = m_remoteUserStream.find(titleName);
    if (remoteUserStreamIt == m_remoteUserStream.end())
    {
        qDebug() << "[" << __FUNCTION__ << "] has not published stream";
        return;
    }

    DisplayWidget* widget = getIdleDisplayWidget();

    if (!widget)
    {
        qDebug("[%s] all the Windows are full! ignore.", __FUNCTION__);
        return;
    }

    StreamParam streamParam;
    SubscribeStreamParam subParam;
    auto& userStream = remoteUserStreamIt.value();
    if (userStream.find(STREAM_TYPE_MAIN_VIDEO) != userStream.end())
    {
        streamParam = userStream[STREAM_TYPE_MAIN_VIDEO];

        subParam.type = streamParam.type;
        subParam.layerIndex = streamParam.layerCount - 1;
        subParam.mediaIndex = streamParam.mediaIndex;
        qDebug("[%s] subscribeStream userId:%s, stream type:%d", __FUNCTION__, titleName.toStdString().c_str(), streamParam.type);
        this->m_client->subscribeStream(titleName.toStdString().c_str(), &subParam, widget->getWinId());
        widget->subscribeStream(titleName, streamParam.type,streamParam.mediaIndex);
    }
    else if (userStream.find(STREAM_TYPE_DUAL_VIDEO) != userStream.end())
    {
        streamParam = userStream[STREAM_TYPE_DUAL_VIDEO];

        subParam.type = streamParam.type;
        subParam.layerIndex = streamParam.layerCount - 1;
        subParam.mediaIndex = streamParam.mediaIndex;
        int idx = titleName.lastIndexOf("的共享内容");
        QString id = titleName.mid(0, idx); //截取出真正的userid
        qDebug("[%s] subscribeStream titleName:%s, userId:%s, stream type:%d", __FUNCTION__, titleName.toStdString().c_str(), id.toStdString().c_str(), streamParam.type);
        this->m_client->subscribeStream(id.toStdString().c_str(), &subParam, widget->getWinId());
        widget->subscribeStream(id, streamParam.type,streamParam.mediaIndex);
    }
    else if (userStream.find(STREAM_TYPE_VMP) != userStream.end())
    {
        streamParam = userStream[STREAM_TYPE_VMP];

        subParam.type = streamParam.type;
        subParam.layerIndex = streamParam.layerCount - 1;
        subParam.mediaIndex = streamParam.mediaIndex;
        qDebug("[%s] subscribeStream userId:%s, stream type:%d", __FUNCTION__, titleName.toStdString().c_str(), streamParam.type);
        this->m_client->subscribeStream("", &subParam, widget->getWinId());
        widget->subscribeStream("", streamParam.type,streamParam.mediaIndex);
    }
}

void RoomWindow::on_btnSetup_clicked()
{
    qDebug() << __FUNCTION__;
    DeviceListWindow* w = new DeviceListWindow(this);
    w->open();
}

void RoomWindow::on_mixSwitchCheckBox_stateChanged(int state)
{
    qDebug("[%s] checkbox state:  %d", __FUNCTION__, state);

    this->changeAudioPattern(state);

}

void RoomWindow::changeAudioPattern(int state)
{
    //取消订阅混音
    if(state == Qt::CheckState::Unchecked)
    {
        if(this->hasSubscribedMix == false)
        {
            qDebug("[%s]want close mix but not Subscribed Mix, return.", __FUNCTION__);
            return;
        }

        userId mixId = {};
        this->m_client->unsubscribeStream(mixId,STREAM_TYPE_MIX_AUDIO,0);
        //此处sdk连续取消订阅
        for (int i = 0; i < 3; ++i)
        {
            QString strId = this->m_view[i]->getSubscribeUser();
            auto it = m_remoteUserStream.find(strId);
            if(it != this->m_remoteUserStream.end() && it.value().contains(STREAM_TYPE_AUDIO))
            {
                SubscribeStreamParam subParam;
                subParam.type = STREAM_TYPE_AUDIO;
                this->m_client->subscribeStream(strId.toStdString().c_str(), &subParam, nullptr);
            }
        }

        this->hasSubscribedMix = false;
    }
    //订阅混音
    else if(state == Qt::CheckState::Checked)
    {
        if(this->hasSubscribedMix == true)
        {
            qDebug("[%s]want open mix and has Subscribed Mix, return.", __FUNCTION__);
            return;
        }

        for (int i = 0; i < 3; ++i)
        {
            QString strId = this->m_view[i]->getSubscribeUser();
            auto it = m_remoteUserStream.find(strId);
            if(it != this->m_remoteUserStream.end() && it.value().contains(STREAM_TYPE_AUDIO))
            {
                this->m_client->unsubscribeStream(strId.toStdString().c_str(), STREAM_TYPE_AUDIO, 0);
            }
        }
        SubscribeStreamParam subParam;
        subParam.type = STREAM_TYPE_MIX_AUDIO;
        userId mixId = {};
        this->m_client->subscribeStream(mixId, &subParam, nullptr);
        this->hasSubscribedMix = true;
    }
}

void RoomWindow::onSigUnpublishStreamNotify(STREAM_TYPE type, uint8_t mediaIndex)
{
    qDebug("[%s]notify unpublish stream type:%d, mediaIndex:%hhu", __FUNCTION__, type, mediaIndex);
    return;
}

void RoomWindow::onSigSnapshotVideo(QString id, STREAM_TYPE type, uint8_t mediaIndex)
{
    char* ch;
    QByteArray ba = id.toLatin1();
    ch = ba.data();
    m_client->snapshotVideo(ch,type,mediaIndex);
}

void RoomWindow::onSigSaveImage(QImage image)
{
    QString filepath =  QCoreApplication::applicationDirPath() + "/" + QDateTime::currentDateTime().toString("yyMMddhhmmss") + ".bmp";
    image.save(filepath);
    QMessageBox::about(NULL,"图片路径",filepath);
}

void RoomWindow::unpublishStreamHandle(STREAM_TYPE type)
{
    if (type == STREAM_TYPE_MAIN_VIDEO)
    {
        ui->camera->setStyleSheet("QPushButton{ border-image: url(\":/icon/camera-close.png\"); }"
                                  "QPushButton:pressed { border-image: url(\":/icon/camera-close-select.png\"); } }");
        this->cameraEnable = false;
        ui->camera->setEnabled(true);
    }
    else if (type == STREAM_TYPE_DUAL_VIDEO)
    {
        ui->screenShare->setStyleSheet("QPushButton{ border-image: url(\":/icon/screen-share-close.png\"); }"
                                       "QPushButton:pressed { border-image: url(\":/icon/screen-share-close-select.png\"); }");
        this->screenShare = false;
        ui->screenShare->setEnabled(true);
    }
    else if (type == STREAM_TYPE_AUDIO)
    {
    }
}

